using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;
using ruche.aviutl.plugins;

namespace ruche.nive.aviutlPluginPorts
{
    /// <summary>
    /// AviUtl̓vOC𗘗pNiVEI[fBI̓vOCNXB
    /// MediaInput ō쐬B
    /// </summary>
    internal class AudioMediaInput : IMediaInput, IInputChild
    {
        /// <summary>
        /// eCX^XB
        /// </summary>
        private IMediaInput _parent;

        /// <summary>
        /// ̓vOCC^tF[XB
        /// </summary>
        private InputPlugin _plugin;

        /// <summary>
        /// vWFNgB
        /// </summary>
        private IHost _host;

        /// <summary>
        /// ʔԍ
        /// </summary>
        private long _inputCode;

        /// <summary>
        /// RXgN^B
        /// </summary>
        /// <param name="parent">eCX^XB</param>
        /// <param name="plugin">
        /// fBAt@Cǂݍݍς݂̓̓vOCB
        /// </param>
        /// <param name="host">vWFNgB</param>
        /// <param name="id">q̎IDB</param>
        public AudioMediaInput(
            IMediaInput parent, InputPlugin plugin, IHost host, int id)
        {
            if (parent == null)
            {
                throw new NullReferenceException("parent  null łB");
            }
            if (plugin == null)
            {
                throw new NullReferenceException("plugin  null łB");
            }
            if (host == null)
            {
                throw new NullReferenceException("host  null łB");
            }
            if (!plugin.IsMediaFileOpened)
            {
                throw new ArgumentException(
                    "fBAt@CJĂ܂B", "plugin");
            }

            _parent = parent;
            _plugin = plugin;
            _host = host;
            _inputCode =
                ((long)_plugin.MediaFilePath.GetHashCode() << 32) + id;
        }

        /// <summary>
        /// ͂̊JnsB
        /// </summary>
        void BeginInput()
        {
            // vOĈfBNgֈړ
            CurrentDirectoryStack.Instance.Push(
                Path.GetDirectoryName(_plugin.PluginFilePath));
        }

        /// <summary>
        /// ͂̏IsB
        /// </summary>
        void EndInput()
        {
            // JgfBNg
            CurrentDirectoryStack.Instance.Pop();
        }

        #region IMediaInput o

        public string PluginName
        {
            get { return "AviUtl̓vOCp"; }
        }

        public string CorrespondingType
        {
            get { return "fBAt@C (*.*)|*.*"; }
        }

        public string FileName
        {
            get { return _plugin.MediaFilePath; }
        }

        public void Initialize(IHost host)
        {
            // gpȂ
        }

        public bool LoadFile(string file)
        {
            // gpȂ
            return false;
        }

        public MediaInfo GetMediaInfo()
        {
            int count = _plugin.MediaFileInfo.InputAudioSampleCount;

            return new MediaInfo(
                count * ReferenceData.WaveFormat.nBlockAlign,
                Size.Empty, MediaType.Audio, 0.0);
        }

        public int GetLength()
        {
            int count = _plugin.MediaFileInfo.InputAudioSampleCount;

            // TvTv[g~t[[gt[
            return (int)(
                (count * _host.ProjectFrameRate) /
                ReferenceData.WaveFormat.nSamplesPerSec);
        }

        public Bitmap GetFrame(int index)
        {
            return PluginUtils.NullImage(1, 1);
        }

        public byte[] GetAudio()
        {
            return null;
        }

        public byte[] GetFrameAudio(int index)
        {
            byte[] data = null;

            BeginInput();

            // t[ʒut[[g~Tv[gTvʒu
            uint sampleRate = ReferenceData.WaveFormat.nSamplesPerSec;
            int start =
                (int)((index / _host.ProjectFrameRate) * sampleRate);
            int next =
                (int)(((index + 1) / _host.ProjectFrameRate) * sampleRate);
            int length = next - start;

            try
            {
                // f[^擾
                data = _plugin.ReadMediaFileAudio(start, length);
            }
            finally
            {
                EndInput();

                if (data == null)
                {
                    data = new byte[
                        length * ReferenceData.WaveFormat.nBlockAlign];
                    Array.Clear(data, 0, data.Length);
                }
            }

            return data;
        }

        #endregion

        #region IDisposable o

        public void Dispose()
        {
            _plugin = null;
        }

        #endregion

        #region IInputChild o

        public long InputCode
        {
            get { return _inputCode; }
        }

        public IMediaInput Parent
        {
            get { return _parent; }
        }

        #endregion
    }
}
