using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Drawing;
using nicoAE.Plugin.Interface;
using nicoAE.Plugin.Utils;
using ruche.aviutl.plugins;

namespace ruche.nive.aviutlPluginPorts
{
    /// <summary>
    /// AviUtl̓vOC𗘗pNiVE̓vOCNXB
    /// </summary>
    public class MediaInput : IMediaInput2
    {
        /// <summary>
        /// vOCꗗ̃L[B
        /// </summary>
        private static readonly string PluginFileListKeyName =
            typeof(MediaInput).Name + ":PluginFileList";

        /// <summary>
        /// ̓vOCݒ_CAO\tOt@C̃tpXB
        /// </summary>
        private static readonly string DialogNotShownFlagFilePath =
            Path.Combine(
                Application.StartupPath,
                typeof(MediaInput).FullName + ".NODIALOG");

        /// <summary>
        /// vWFNgB
        /// </summary>
        private IHost _host = null;

        /// <summary>
        /// ̓vOC̃C^tF[XB
        /// </summary>
        private InputPlugin _plugin = null;

        /// <summary>
        /// ̓C^tF[XzB
        /// </summary>
        List<IMediaInput> _mediaInputs = new List<IMediaInput>();

        /// <summary>
        /// ̓vOCݒ_CAO\ȂݒɂȂĂ邩
        /// ۂ擾B
        /// </summary>
        /// <returns>
        /// \ȂݒɂȂĂȂ true AłȂ false B
        /// </returns>
        static bool IsDialogNotShown()
        {
            return File.Exists(DialogNotShownFlagFilePath);
        }

        /// <summary>
        /// ̓vOCݒ_CAO̔\ݒL܂͖ɂB
        /// </summary>
        /// <param name="notShown">
        /// \ɂȂ true AłȂ false B
        /// </param>
        static void SetDialogNotShown(bool notShown)
        {
            if (notShown)
            {
                File.Create(DialogNotShownFlagFilePath).Close();
            }
            else
            {
                File.Delete(DialogNotShownFlagFilePath);
            }
        }

        /// <summary>
        /// fBAt@C̓vOCt@CQœǂݎ莎sA
        /// ̓vOCC^tF[XԂB
        /// </summary>
        /// <param name="mediaFile">fBAt@C̃pXB</param>
        /// <param name="pluginFiles">̓vOCt@C̃pXzB</param>
        /// <returns>̓vOCC^tF[XBsꍇ null B</returns>
        static InputPlugin OpenMediaFileForEach(
            string mediaFile, string[] pluginFiles)
        {
            InputPlugin plugin = null;

            foreach (string pf in pluginFiles)
            {
                try
                {
                    // JgfBNgvOC̏ꏊɂ
                    CurrentDirectoryStack.Instance.Push(
                        Path.GetDirectoryName(pf));

                    // ǂݎ
                    plugin = new InputPlugin(pf);
                    if (plugin.OpenMediaFile(mediaFile))
                    {
                        // 
                        break;
                    }

                    plugin.Dispose();
                    plugin = null;
                }
                catch
                {
                    if (plugin != null)
                    {
                        plugin.Dispose();
                        plugin = null;
                    }
                }
                finally
                {
                    CurrentDirectoryStack.Instance.Pop();
                }
            }

            return plugin;
        }

        /// <summary>
        /// JĂ郁fBAt@CB
        /// </summary>
        void CloseMediaFile()
        {
            if (_mediaInputs != null)
            {
                foreach (IMediaInput mi in _mediaInputs)
                {
                    mi.Dispose();
                }
                _mediaInputs.Clear();
            }

            if (_plugin != null)
            {
                try
                {
                    // JgfBNgvOC̏ꏊɂ
                    CurrentDirectoryStack.Instance.Push(
                        Path.GetDirectoryName(_plugin.PluginFilePath));

                    _plugin.Dispose();
                    _plugin = null;
                }
                finally
                {
                    CurrentDirectoryStack.Instance.Pop();
                }
            }
        }

        #region IMediaInput2 o

        public IMediaInput[] GetInputs()
        {
            return (_mediaInputs == null) ? null : _mediaInputs.ToArray();
        }

        #endregion

        #region IMediaInput o

        public string PluginName
        {
            get { return "AviUtl̓vOCp"; }
        }

        public string CorrespondingType
        {
            get { return "fBAt@C (*.*)|*.*"; }
        }

        public string FileName
        {
            get { return (_plugin == null) ? "" : _plugin.MediaFilePath; }
        }

        public void Initialize(IHost host)
        {
            // vWFNgݒ
            _host = host;

            // Cuݒ̃[h
            LibraryConfig.Datas.Load();
        }

        public bool LoadFile(string file)
        {
            bool success = false;

            // Ƃ肠
            CloseMediaFile();
            _plugin = null;

            // vOCꗗݒǂݎ
            string[] pluginFiles =
                LibraryConfig.Datas[PluginFileListKeyName] as string[];

            if (!IsDialogNotShown())
            {
                bool cancel = false;

                // t@C񃊃Xg쐬
                List<FileInfo> pluginInfos = null;
                if (pluginFiles != null)
                {
                    pluginInfos = new List<FileInfo>(pluginFiles.Length);
                    foreach (string pf in pluginFiles)
                    {
                        pluginInfos.Add(new FileInfo(pf));
                    }
                }

                using (MediaInputConfigDialog dlg = new MediaInputConfigDialog())
                {
                    // lݒ
                    dlg.MediaFilePath = file;
                    if (pluginInfos != null)
                    {
                        dlg.PluginFiles = pluginInfos;
                    }

                    // _CAO\
                    cancel = (dlg.ShowDialog() != DialogResult.OK);
                    if (!cancel)
                    {
                        // fBAt@CpXXV
                        file = dlg.MediaFilePath;

                        // vOCꗗXV
                        pluginFiles = new string[dlg.PluginFiles.Count];
                        for (int i = 0; i < pluginFiles.Length; ++i)
                        {
                            pluginFiles[i] = dlg.PluginFiles[i].FullName;
                        }

                        // ݒt@C֕ۑ
                        LibraryConfig.Datas[PluginFileListKeyName] = pluginFiles;
                        LibraryConfig.Datas.Save();

                        // \ݒ蔽f
                        SetDialogNotShown(dlg.IsNotShownChecked);
                    }
                }

                if (cancel) { return false; }
            }

            // ǂݎ莎s
            if (pluginFiles != null)
            {
                _plugin = OpenMediaFileForEach(file, pluginFiles);
            }
            if (_plugin != null)
            {
                InputMediaFlags types = _plugin.MediaFileInfo.InputTypes;
                if ((types & InputMediaFlags.Video) != 0)
                {
                    // rfI͍쐬
                    _mediaInputs.Add(
                        new VideoMediaInput(this, _plugin, _host, 0));
                }
                if ((types & InputMediaFlags.Audio) != 0)
                {
                    // I[fBI͍쐬
                    _mediaInputs.Add(
                        new AudioMediaInput(this, _plugin, _host, 1));
                }

                success = (_mediaInputs.Count > 0);
            }

            return success;
        }

        public MediaInfo GetMediaInfo()
        {
            return new MediaInfo();
        }

        public int GetLength()
        {
            return 0;
        }

        public Bitmap GetFrame(int index)
        {
            return PluginUtils.NullImage(1, 1);
        }

        public byte[] GetAudio()
        {
            return null;
        }

        public byte[] GetFrameAudio(int index)
        {
            return null;
        }

        #endregion

        #region IDisposable o

        public void Dispose()
        {
            CloseMediaFile();
        }

        #endregion
    }
}
