using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using ruche.aviutl.plugins;

namespace ruche.nive.aviutlPluginPorts
{
    /// <summary>
    /// AviUtlo̓vOCɂo͏s_CAOB
    /// </summary>
    public partial class AviutlOutputDialog : Form
    {
        /// <summary>
        /// vr[\L̃L[B
        /// </summary>
        private readonly string PreviewEnabledKeyName =
            typeof(AviutlOutputDialog).Name + ":PreviewEnabled";

        /// <summary>
        /// _CAOLvṼx[XB
        /// </summary>
        private const string CaptionBase = "AviUtlo̓vOCp";

        private OutputPlugin _plugin = null;
        private IOutputInfo _info = null;
        private IAbortable _aborter = null;
        private IHavePreviewImage _hasPreview = null;

        private Thread _thread = null;
        private bool _curDirPushed = false;

        /// <summary>
        /// RXgN^łB
        /// </summary>
        /// <param name="plugin">o̓vOCC^tF[XB</param>
        /// <param name="info">o͏B</param>
        /// <param name="aborter">A{[g񋟁B</param>
        /// <param name="hasPreview">vr[񋟁B null ł悢B</param>
        public AviutlOutputDialog(
            OutputPlugin plugin,
            IOutputInfo info,
            IAbortable aborter,
            IHavePreviewImage hasPreview)
        {
            if (info == null)
            {
                throw new ArgumentNullException("info");
            }
            if (aborter == null)
            {
                throw new ArgumentNullException("aborter");
            }

            _plugin = plugin;
            _info = info;
            _aborter = aborter;
            _hasPreview = hasPreview;

            InitializeComponent();
        }

        /// <summary>
        /// vr[̃TCYɉă_CAOTCY܂B
        /// </summary>
        /// <param name="rate">vr[̕\{B</param>
        /// <returns>
        /// TCYɐȂ true BłȂ false B
        /// </returns>
        public bool ResizeByPreview(double rate)
        {
            bool result = false;

            try
            {
                if (_hasPreview != null && _hasPreview.PreviewImage != null)
                {
                    // TCYݒ
                    Size size = _hasPreview.PreviewImage.Size;
                    size.Width = (int)(size.Width * rate);
                    size.Height = (int)(size.Height * rate);

                    if (size.Width > 0 && size.Height > 0)
                    {
                        // ݂̃sN`{bNXTCYƂ̍𓾂
                        int dw = size.Width - picturePreview.Width;
                        int dh = size.Height - picturePreview.Height;

                        // TCYύX
                        this.Width += dw;
                        this.Height += dh;

                        result = true;
                    }
                }
            }
            catch
            {
                result = false;
            }

            return result;
        }

        private delegate void SetControlTextDelegate(Control c, string text);

        /// <summary>
        /// XbhZ[tȃRg[̃eLXgݒsB
        /// </summary>
        /// <param name="c">ΏۃRg[B</param>
        /// <param name="text">eLXgB</param>
        private void SetControlText(Control c, string text)
        {
            if (InvokeRequired)
            {
                SetControlTextDelegate proc =
                    new SetControlTextDelegate(SetControlText);
                Invoke(proc, c, text);
            }
            else
            {
                c.Text = text;
            }
        }

        private delegate void UpdatePreviewImageDelegate();

        /// <summary>
        /// XbhZ[tȃvr[摜̍XVsB
        /// </summary>
        private void UpdatePreviewImage()
        {
            if (InvokeRequired)
            {
                UpdatePreviewImageDelegate proc =
                    new UpdatePreviewImageDelegate(UpdatePreviewImage);
                Invoke(proc);
            }
            else if (_hasPreview != null)
            {
                Image img = _hasPreview.PreviewImage;
                if (img != null)
                {
                    picturePreview.Image = img;
                    picturePreview.SizeMode = PictureBoxSizeMode.Zoom;
                }
            }
        }

        private delegate void ShowMessageBoxDelegate(
            string text, string caption, MessageBoxButtons buttons);

        /// <summary>
        /// XbhZ[tȃbZ[W{bNX̕\sB
        /// </summary>
        /// <param name="text">eLXgB</param>
        /// <param name="caption">LvVB</param>
        /// <param name="buttons">{^B</param>
        private void ShowMessageBox(
            string text, string caption, MessageBoxButtons buttons)
        {
            if (InvokeRequired)
            {
                ShowMessageBoxDelegate proc =
                    new ShowMessageBoxDelegate(ShowMessageBox);
                Invoke(proc, text, caption, buttons);
            }
            else
            {
                MessageBox.Show(this, text, caption, buttons);
            }
        }

        /// <summary>
        /// o̓XbhsB
        /// </summary>
        private void OutputThread()
        {
            bool ok = false;

            // ̎s
            try
            {
#if OUTPUT_WORK_TIME
                // Jn
                DateTime startTime = DateTime.Now;
#endif // OUTPUT_WORK_TIME

                ok = _plugin.Execute(_info);

#if OUTPUT_WORK_TIME
                // vvԂt@Cɏo
                TimeSpan span = DateTime.Now - startTime;
                File.AppendAllText(
                    _info.OutputFilePath + "_worktime.txt",
                    "OUTPUT TIME : " + span.ToString() + "\n",
                    Encoding.Default);
#endif // OUTPUT_WORK_TIME
            }
            catch (Exception ex)
            {
                if (!_info.IsOutputAborted)
                {
                #if DEBUG
                    ShowMessageBox(
                        ex.Message + "\n\n" + ex.StackTrace, ex.GetType().FullName,
                        MessageBoxButtons.OK);
                #else
                    ShowMessageBox(ex.Message, "G[", MessageBoxButtons.OK);
                #endif
                }

                ok = false;
            }

            if (!_info.IsOutputAborted)
            {
                // Rg[ύX
                string resText = ok ? "" : "s";
                SetControlText(labelProgress, resText);
                SetControlText(buttonClose, "(&C)");
                SetControlText(this, "[" + resText + "] " + CaptionBase);
            }
        }

        private void AviutlOutputDialog_Load(object sender, EventArgs e)
        {
            if (_hasPreview == null)
            {
                // vr[񋟂΃vr[
                checkPreview.Checked = false;
                checkPreview.Visible = false;
            }
            else if (LibraryConfig.Datas.ContainsKey(PreviewEnabledKeyName))
            {
                // EBhETCY
                ResizeByPreview(1.0);

                // ݒ肪Γǂݍ
                try
                {
                    checkPreview.Checked =
                        (bool)LibraryConfig.Datas[PreviewEnabledKeyName];
                }
                catch { }
            }

            // fQ[go^
            _info.FrameProgress +=
                new FrameProgressDelegate(UpdateFrameProgress);
            _info.UpdatePreview +=
                new UpdatePreviewDelegate(UpdatePreview);

            // JgfBNgvOC̏ꏊɂ
            CurrentDirectoryStack.Instance.Push(
                Path.GetDirectoryName(_plugin.PluginFilePath));
            _curDirPushed = true;

            // XbhJn
            _thread = new Thread(OutputThread);
            _thread.Start();

            buttonClose.Enabled = true;
        }

        private void AviutlOutputDialog_FormClosing(
            object sender, FormClosingEventArgs e)
        {
            if (!buttonClose.Enabled)
            {
                // ܂n܂ĂȂ
                e.Cancel = true;
                DialogResult = DialogResult.None;
            }
            else if (_thread.IsAlive)
            {
                // ܂Xbhғ
                DialogResult res = MessageBox.Show(
                    this, "o͂𒆒f܂H", "̒f",
                    MessageBoxButtons.YesNo);
                if (res == DialogResult.Yes)
                {
                    try
                    {
                        // A{[g
                        buttonClose.Enabled = false;
                        _aborter.Abort();

                        // XbhI܂ő҂
                        while (!_thread.Join(10))
                        {
                            // WindowsbZ[W
                            Application.DoEvents();
                        }
                    }
                    finally
                    {
                        buttonClose.Enabled = true;
                        DialogResult = DialogResult.Cancel;
                    }
                }
                else
                {
                    e.Cancel = true;
                    DialogResult = DialogResult.None;
                }
            }
            else
            {
                // Xbh͊ɏIĂ
                DialogResult = DialogResult.OK;
            }

            if (DialogResult != DialogResult.None)
            {
                if (_hasPreview != null)
                {
                    // vr[\ݒۑ
                    LibraryConfig.Datas[PreviewEnabledKeyName] =
                        checkPreview.Checked;
                    try
                    {
                        LibraryConfig.Datas.Save();
                    }
                    catch { }
                }

                // JgfBNg𕜌
                if (_curDirPushed)
                {
                    CurrentDirectoryStack.Instance.Pop();
                    _curDirPushed = false;
                }
            }
        }

        private bool UpdateFrameProgress(int frames, int totalFrames)
        {
            string progress = "" + frames + "/" + totalFrames;
            SetControlText(labelProgress, progress);
            SetControlText(this, "[" + progress + "] " + CaptionBase);
            return true;
        }

        private bool UpdatePreview()
        {
            if (_hasPreview != null && checkPreview.Checked)
            {
                UpdatePreviewImage();
            }
            return true;
        }

        private void cmenuResizeX05_Click(object sender, EventArgs e)
        {
            ResizeByPreview(0.5);
        }

        private void cmenuResizeX10_Click(object sender, EventArgs e)
        {
            ResizeByPreview(1.0);
        }

        private void cmenuResizeX20_Click(object sender, EventArgs e)
        {
            ResizeByPreview(2.0);
        }
    }
}